<?php

class NOTIFICATIONS_BOL_Service
{
    const NOTIFY_TYPE_IMMEDIATELY = 'immediately';
    const NOTIFY_TYPE_DAILY = 'daily';
    const NOTIFY_TYPE_WEEKLY = 'weekly';

    private static $classInstance;

    /**
     * Returns class instance
     *
     * @return NOTIFICATIONS_BOL_Service
     */
    public static function getInstance()
    {
        if ( null === self::$classInstance )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     *
     * @var NOTIFICATIONS_BOL_RuleDao
     */
    private $ruleDao;

    /**
     *
     * @var NOTIFICATIONS_BOL_UnsubscribeDao
     */
    private $unsubscribeDao;

    /**
     *
     * @var NOTIFICATIONS_BOL_NotificationDao
     */
    private $notificationDao;

    private $defaultRuleList = array();

    public function __construct()
    {
        $this->ruleDao = NOTIFICATIONS_BOL_RuleDao::getInstance();
        $this->unsubscribeDao = NOTIFICATIONS_BOL_UnsubscribeDao::getInstance();
        $this->notificationDao = NOTIFICATIONS_BOL_NotificationDao::getInstance();
    }

    public function collectActionList()
    {
        if ( empty($this->defaultRuleList) )
        {
            $event = new BASE_CLASS_EventCollector('notifications.collect_actions');
            OW::getEventManager()->trigger($event);

            $eventData = $event->getData();
            foreach ( $eventData as $item )
            {
                $this->defaultRuleList[$item['action']] = $item;
            }
        }

        return $this->defaultRuleList;
    }

    public function findRuleList( $userId, $actions = null )
    {
        $out = array();
        $list = $this->ruleDao->findRuleList($userId, $actions);
        foreach ( $list as $item )
        {
            $out[$item->action] = $item;
        }

        return $out;
    }

    public function saveRule( NOTIFICATIONS_BOL_Rule $rule )
    {
        $this->ruleDao->save($rule);
    }

    public function findUserIdListForSend( $count, $period = null )
    {
        if ( empty($period) )
        {
            $period = 24 * 3600;
        }

        return $this->notificationDao->findUserIdListForSend($count, $period);
    }

    public function findUserIdByUnsubscribeCode( $code )
    {
        $dto = $this->unsubscribeDao->findByCode($code);

        return  empty($dto) ? null : $dto->userId;
    }

    private function getUnsubscribeCodeLifeTime()
    {
        return 60 * 60 * 24 * 7;
    }

    public function deleteExpiredUnsubscribeCodeList()
    {
        $time = time() - $this->getUnsubscribeCodeLifeTime();
        $this->unsubscribeDao->deleteExpired($time);
    }

    public function generateUnsubscribeCode( BOL_User $user )
    {
        $code = md5($user->email);
        $dto = new NOTIFICATIONS_BOL_Unsubscribe();
        $dto->userId = $user->id;
        $dto->code = $code;
        $dto->timeStamp = time();

        $this->unsubscribeDao->save($dto);

        return $code;
    }

    public function sendPermittedNotifications( $userId, $notificationList )
    {
        $defaultRules = $this->collectActionList();
        $rules = $this->findRuleList($userId);

        $listToSend = array();
        foreach ( $notificationList as $notification )
        {
            $action = $notification['action'];

            if ( isset($rules[$action]) )
            {
                if ( !$rules[$action]->checked )
                {
                    continue;
                }
            }
            else
            {
                if ( empty($defaultRules[$action]['selected']) )
                {
                    continue;
                }
            }

            $listToSend[] = $notification;
        }

        $this->sendNotifications($userId, $listToSend, false);
    }

    public function sendNotifications( $userId, $notifications )
    {
        if ( empty($notifications) )
        {
            return;
        }

        $cmp = new NOTIFICATIONS_CMP_Notification($userId);

        foreach ( $notifications as $item )
        {
            $data = $item['data'];
            $params = $item;
            $onEvent = new OW_Event('notifications.on_item_send', $params, $data);
            OW::getEventManager()->trigger($onEvent);

            $item['data'] = $onEvent->getData();

            $cmp->addItem($item);
        }

        $this->sendProcess($userId, $cmp);
    }

    private function sendProcess( $userId, NOTIFICATIONS_CMP_Notification $cmp )
    {
        $userService = BOL_UserService::getInstance();
        $user = $userService->findUserById($userId);

        if ( empty($user) )
        {
            return false;
        }

        $email = $user->email;
        $unsubscribeCode = $this->generateUnsubscribeCode($user);

        $cmp->setUnsubscribeCode($unsubscribeCode);

        $txt = $cmp->getTxt();
        $html = $cmp->getHtml();

        $subject = $cmp->getSubject();

        try
        {
            $mail = OW::getMailer()->createMail()
                ->addRecipientEmail($email)
                ->setTextContent($txt)
                ->setHtmlContent($html)
                ->setSubject($subject);

            OW::getMailer()->send($mail);
        }
        catch ( Exception $e )
        {
            //Skip invalid notification
        }
    }



    public function findNotificationList( $userId, $beforeStamp, $ignoreIds, $count )
    {
        return $this->notificationDao->findNotificationList($userId, $beforeStamp, $ignoreIds, $count);
    }

    public function findNotificationListForSend( $userIdList )
    {
        return $this->notificationDao->findNotificationListForSend($userIdList);
    }

    public function findNotificationCount( $userId, $viewed = null )
    {
        return $this->notificationDao->findNotificationCount($userId, $viewed);
    }

    public function saveNotification( NOTIFICATIONS_BOL_Notification $notification )
    {
        $this->notificationDao->saveNotification($notification);
    }

    /**
     *
     * @param string $entityType
     * @param int $entityId
     * @param int $userId
     * @return NOTIFICATIONS_BOL_Notification
     */
    public function findNotification( $entityType, $entityId, $userId )
    {
        return $this->notificationDao->findNotification($entityType, $entityId, $userId);
    }

    public function markNotificationsViewedByIds( $idList, $viewed = true )
    {
        $this->notificationDao->markViewedByIds($idList, $viewed);
    }

    public function markNotificationsSentByIds( $idList, $sent = true )
    {
        $this->notificationDao->markSentByIds($idList, $sent);
    }

    public function deleteNotification( $entityType, $entityId, $userId )
    {
        $this->notificationDao->deleteNotification($entityType, $entityId, $userId);
    }

    public function deleteExpiredNotification()
    {
        $this->notificationDao->deleteExpired();
    }

    public function deleteNotificationByEntity( $entityType, $entityId )
    {
        $this->notificationDao->deleteNotificationByEntity($entityType, $entityId);
    }

    public function deleteNotificationByPluginKey( $pluginKey )
    {
        $this->notificationDao->deleteNotificationByPluginKey($pluginKey);
    }

    public function setNotificationStatusByPluginKey( $pluginKey, $status )
    {
        $this->notificationDao->setNotificationStatusByPluginKey($pluginKey, $status);
    }
}
